#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# 模块重写脚本

from inspect import getmodule
import os
from unicodedata import name
from ruamel import yaml
from tdf_tools.dependencies_analysis import DependencyAnalysis, DependencyNode
from tdf_tools.tdf_print import printStage, printError, printStr
from enum import Enum
from tdf_tools.json_data_read import getInitJsonData, getModuleJsonData, getModuleNameList, projectModuleDir, REWRITE_TYPE


class ModuleDependenciesRewriteUtil(object):

    def __init__(self):
        self.moduleJsonData = getModuleJsonData()
        self.moduleNameList = getModuleNameList()
        self.initJsonData = getInitJsonData()

    # 分析lock文件，获取所有的packages
    def _analysisLock(self):
        os.chdir(self.__moduleGenPath)
        # 分析前先执行pub upgrade
        os.popen('flutter pub upgrade').read()

        # 读取lock内容
        with open('pubspec.lock', encoding='utf-8') as f:
            doc = yaml.round_trip_load(f)
            if (isinstance(doc, dict) and doc.__contains__('packages')):
                f.close()
                return doc['packages']

    # 是否是壳模块
    def _isShellModule(self):
        return self.moduleJsonData[self.__moduleName]['type'] == 'app'

    # 确认哪些依赖需要重写
    def _confirmRewriteDependencies(self):
        if self._isShellModule():  # 壳模块重写所有配置的依赖
            for item in self.moduleNameList:
                if (item != self.__moduleName):  # 壳自己不加入重写列表
                    self.__needRewriteDependencies.append(item)
        else:  # 如果不是壳模块，则进行lock文件内的package列表和开发模块匹配，匹配上则添加到override列表
            for package in self.__moduleDependenciesMap:
                for module in self.moduleNameList:
                    if package == module:
                        self.__needRewriteDependencies.append(module)

        printStage("{0}中以下依赖将被override".format(self.__moduleName))
        printStr(self.__needRewriteDependencies)

    def _addOverrideDependencies(self, type):
        mDict = dict()
        if type == REWRITE_TYPE.LOCAL:
            for item in self.__needRewriteDependencies:
                mDict[item] = {
                    "path": "../{0}/".format(item)}
        # feature和test类型的重写，暂时不会被用到
        elif type == REWRITE_TYPE.FEATURE:
            for item in self.__needRewriteDependencies:
                mDict[item] = {
                    "git": {"url": self.moduleJsonData[item]['git'], "ref": self.initJsonData['featureBranch']}}
        elif type == REWRITE_TYPE.TEST:
            for item in self.__needRewriteDependencies:
                mDict[item] = {
                    "git": {"url": self.moduleJsonData[item]['git'], "ref": self.initJsonData['testBranch']}}

        return mDict

    # 添加dependency_overrides

    def _rewriteDependencies(self, type):
        os.chdir(self.__moduleGenPath)
        with open('pubspec.yaml', encoding='utf-8') as f:
            doc = yaml.round_trip_load(f)
            if isinstance(doc, dict):
                if doc.__contains__('dependency_overrides') and doc['dependency_overrides'] != None:
                    # printError(
                    #     "dependency_overrides遗留，发现开发不规范，强制删除dependency_overrides", False)
                    # printStr(doc['dependency_overrides'])
                    doc['dependency_overrides'] = None

                self._confirmRewriteDependencies()
                overrideDict = self._addOverrideDependencies(
                    type)
                if bool(overrideDict):
                    doc['dependency_overrides'] = overrideDict

                    open("pubspec.yaml", 'w').close()
                    with open('pubspec.yaml', 'w+', encoding='utf-8') as reW:
                        yaml.round_trip_dump(
                            doc, reW, default_flow_style=False, encoding='utf-8', allow_unicode=True)
                        reW.close()
                        # 依赖重写完，执行pub upgrade更新lock文件
                        os.popen('flutter pub upgrade').read()
                        printStr("lock文件已更新")
            f.close()

    # 重写依赖 本地依赖

    def rewrite(self, type=REWRITE_TYPE.LOCAL):
        for module in self.moduleNameList:
            self.__moduleGenPath = projectModuleDir + "/" + module
            # 这一步会生成lock文件
            self.__moduleDependenciesMap = self._analysisLock()

        # 分析依赖树，从下至上之行upgrade
        dependencyAnalysis = DependencyAnalysis()
        dependencyList = dependencyAnalysis.getDependencyOrder()

        for module in dependencyList:
            self.__moduleName = module
            self.__moduleGenPath = projectModuleDir + "/" + module
            self.__moduleDependenciesMap = self._analysisLock()
            self.__needRewriteDependencies = []
            self._rewriteDependencies(type)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# 模块重写脚本

import os
import json
from enum import Enum
from tdf_tools.tdf_print import printError

# 包含必要的数据校验
# 校验规则：配置的模块列表中必须存在且只存在一个壳应用


class REWRITE_TYPE(Enum):
    LOCAL = 1  # 本地依赖
    FEATURE = 2  # feature分支依赖
    TEST = 3  # test分支依赖


curDir = os.getcwd()

projectGenPath = curDir  # 壳工程根目录
processScriptPath = curDir + '/process/'  # 流程脚本目录
initJsonFile = 'initial_config.json'  # 流程脚本初始化信息配置json文件
moduleJsonFile = 'module_config.json'  # 流程脚本模块git相关信息json文件
# 子module的存放目录，该目录会被 gitignore 忽略
projectModuleDir = curDir + '/.project_module'


def getInitJsonData():  # 获取项目初始化数据
    os.chdir(curDir)
    with open(initJsonFile, 'r', encoding='utf-8') as readF:
        fileData = readF.read()
        readF.close()
        return json.loads(fileData)


def getModuleJsonData():  # 获取模块 git相关配置信息
    os.chdir(curDir)
    with open(moduleJsonFile, 'r', encoding='utf-8') as readF:
        fileData = readF.read()
        readF.close()
        return json.loads(fileData)


def getModuleNameList():
    initJsonData = getInitJsonData()
    if (initJsonData.__contains__('moduleNameList') and isinstance(initJsonData['moduleNameList'], list)):
        moduleNameList = initJsonData['moduleNameList']
        moduleNameList.append(initJsonData['shellName'])
        return moduleNameList
    else:
        printError("❌ 请配置moduleNameList的值,以数组形式")


def branchInvalidate():
    initJsonData = getInitJsonData()
    if (isExistKey(initJsonData, 'featureBranch') and isExistKey(initJsonData, 'testBranch') and initJsonData['featureBranch'] != '' and initJsonData['testBranch'] != '') == False:
        printError("feature和test分支必须配置，详情请参考README.md")


def moduleInvalidate():
    initJsonData = getInitJsonData()
    if initJsonData['shellName'] == "":
        printError("模块校验不通过，请确保：shellName必须配置")


def isExistKey(data, tagkey):
    key_list = data.keys()
    for key in key_list:
        if(key == tagkey):
            return True
    return False



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# 依赖分析【有向无环图，有向有环图】


import json
import os
from ruamel import yaml
import copy
from tdf_tools.tdf_print import printStage, printError, printStr

from tdf_tools.json_data_read import getInitJsonData, getModuleJsonData, getModuleNameList, projectModuleDir, REWRITE_TYPE


class DependencyNode:
    def __init__(self):
        self.nodeName = ''
        self.parent = []  # 父亲节点列表
        self.children = []  # 子孙节点列表
        self.delete = False


class DependencyAnalysis(object):

    def __init__(self):
        printStage("依赖分析")
        self.initJsonData = getInitJsonData()
        self.moduleJsonData = getModuleJsonData()
        self.moduleNameList = getModuleNameList()

    # 分析lock文件，获取所有的packages
    def _analysisLock(self):
        os.chdir(self.__moduleGenPath)
        # 读取lock内容
        with open('pubspec.lock', encoding='utf-8') as f:
            doc = yaml.round_trip_load(f)
            if (isinstance(doc, dict) and doc.__contains__('packages')):
                f.close()
                return doc['packages']

    # 生成依赖图
    def _generateDependenciesMap(self):
        for package in self.__moduleDependenciesMap:
            for module in self.moduleNameList:
                if package == module:
                    # 到这一步表明当前这个模块属于开发模块且在当前模块的依赖模块列表中，是当前模块的子模块
                    self._mNodeDict[self.__moduleName].children.append(package)
                    self._mNodeDict[package].parent.append(self.__moduleName)

    # 返回二维数组，用于并发打tag
    def _generateDependenciesOrder(self):
        resList = []
        while self._existNode():
            itemList = []

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            itemList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

            if len(itemList) == 0:
                break
            resList.append(itemList)
        exit(json.dumps(resList))

    # 返回一维数组，用于从下至上执行upgrade
    def _generateDependenciesOrderForUpgrade(self):
        resList = []
        while self._existNode():
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            resList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

        print(resList)

        return resList

    def _existNode(self):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode) and node.delete == False:
                return True
        return False

    def _subChildCount(self, childName):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode):
                if node.delete == False:
                    if node.children.__contains__(childName):
                        node.children.remove(childName)

    def generate(self):

        self._mNodeDict = dict()
        # 初始化节点列表
        for module in self.moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            self._mNodeDict[module] = node

        # 读取lock文件
        for module in self.moduleNameList:
            self.__moduleName = module
            self.__moduleGenPath = projectModuleDir + "/" + module
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()

        self._generateDependenciesOrder()

    def getDependencyOrder(self):

        self._mNodeDict = dict()
        # 初始化节点列表
        for module in self.moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            self._mNodeDict[module] = node

        # 读取lock文件
        for module in self.moduleNameList:
            self.__moduleName = module
            self.__moduleGenPath = projectModuleDir + "/" + module
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()

        return self._generateDependenciesOrderForUpgrade()
